/*
 *  linux/include/asm-arm/arch-bast/io.h
 *  linux/include/asm-arm/arch-s3c2410/io.h
 *
 *  Copyright (C) 2002 SAMSUNG ELECTRONICS
 *                  SW.LEE <hitchcar@sec.samsung.com>
 *
 * Modifications for BAST:
 *  Copyright (C) 2003 Simtec Electronics
 *       Ben Dooks <ben@simtec.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __ASM_ARM_ARCH_IO_H
#define __ASM_ARM_ARCH_IO_H

#define IO_SPACE_LIMIT 0xffffffff
	/* Used in kernel/resource.c */

/*
 * We have the this routine to use usb host device driver
 *
 */

#define PCI_IO_VADDR      (0x0)
#define PCI_MEMORY_VADDR  (0x0)

/*
 * Generic virtual read/write
 */
#define __arch_getw(a)		(*(volatile unsigned short *)(a))
#define __arch_putw(v,a)	(*(volatile unsigned short *)(a) = (v))

/*
 * Validate and convert memory address for ioremap.
 */
#define iomem_valid_addr(iomem,size)	(1)
#define iomem_to_phys(iomem)	        (iomem)

/*
 *  linux/include/asm-arm/arch-rpc/io.h
 *
 *  Copyright (C) 1997 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Modifications:
 *  06-Dec-1997	RMK	Created.
 *  22-May-2003 BJD     Modified to use with BAST system
 */

/*
 * GCC is totally crap at loading/storing data.  We try to persuade it
 * to do the right thing by using these whereever possible instead of
 * the above.
 */
#define __arch_base_getb(b,o)			\
 ({						\
	unsigned int v, r = (b);		\
	__asm__ __volatile__(			\
		"ldrb	%0, [%1, %2]"		\
		: "=r" (v)			\
		: "r" (r), "Ir" (o));		\
	v;					\
 })

#define __arch_base_getl(b,o)			\
 ({						\
	unsigned int v, r = (b);		\
	__asm__ __volatile__(			\
		"ldr	%0, [%1, %2]"		\
		: "=r" (v)			\
		: "r" (r), "Ir" (o));		\
	v;					\
 })

#define __arch_base_putb(v,b,o)			\
 ({						\
	unsigned int r = (b);			\
	__asm__ __volatile__(			\
		"strb	%0, [%1, %2]"		\
		:				\
		: "r" (v), "r" (r), "Ir" (o));	\
 })

#define __arch_base_putl(v,b,o)			\
 ({						\
	unsigned int r = (b);			\
	__asm__ __volatile__(			\
		"str	%0, [%1, %2]"		\
		:				\
		: "r" (v), "r" (r), "Ir" (o));	\
 })

/* we have several different areas to deal with IO... there is an standard
 * ISA area which is used for the PC104 expansion slot, and the rest is
 * mapped to our IO areas...
 *
 * we take anything below the value of A28 to be offset from the base
 * of the IO base... we may change this later if we don't need to use
 * in/outb with anything other than the 4 fast/slow/byte/word
 */

#define __PORT_PCIO(x)  ((x) < (1<<28))

#include <asm/arch/map.h>

#define PCIO_BASE        (BAST_VA_MULTISPACE)
#define PCIO_BASE_b      (BAST_VA_MULTISPACE | BAST_VAM_CS2)
#define PCIO_BASE_w      (BAST_VA_MULTISPACE | BAST_VAM_CS3)

/* assume we can do outl with 4xoutb */
#define PCIO_BASE_l      (BAST_VA_MULTISPACE)

/*
 * Dynamic IO functions.
 */

/* we assume that any non pcio addresses are automatically correctly
 * qualified for byte/word access at the moment... possibly we need to
 * fix this
*/

#define DECLARE_DYN_OUT(sz,fnsuffix,instr) \
static inline void __out##fnsuffix (unsigned int val, unsigned int port) \
{ \
	unsigned long temp;                   \
        __asm__ __volatile__(                 \
        "cmp    %2, #(1<<28)\n\t"             \
        "mov    %0, %2\n\t" \
        "addcc  %0, %0, %3\n\t"               \
        "str" instr " %1, [ %0, #0 ]"         \
	: "=&r" (temp)                        \
	: "r" (val), "r" (port), "Ir" (PCIO_BASE_##fnsuffix) \
	: "cc"); \
}

DECLARE_DYN_OUT(unsigned char, b, "b")
DECLARE_DYN_OUT(unsigned short, w, "h")
DECLARE_DYN_OUT(unsigned int, l, "")

#define DECLARE_DYN_IN(sz,fnsuffix,instr)			     	\
static inline unsigned sz __in##fnsuffix (unsigned int port)		\
{								      	\
	unsigned long temp, value;					\
	__asm__ __volatile__(						\
	"cmp 	%2, #(1<<28)\n\t"					\
	"mov	%0, %2\n\t"						\
	"addcc	%0, %0, %3\n\t"						\
	"ldr" instr "	%1, [%0, #0 ]	@ in" #fnsuffix		\
	: "=&r" (temp), "=r" (value)					\
	: "r" (port), "Ir" (PCIO_BASE_##fnsuffix)	\
	: "cc");							\
	return (unsigned sz)value;					\
}

static inline unsigned int __ioaddr (unsigned int port)			\
{									\
	if (__PORT_PCIO(port))						\
		return (unsigned int)(PCIO_BASE + (port));		\
	else								\
		return (unsigned int)((port));		\
}

static inline unsigned int __ioaddr_b (unsigned int port)		\
{									\
	if (__PORT_PCIO(port))						\
		return (unsigned int)(PCIO_BASE_b + (port));		\
	else								\
		return (unsigned int)(port);		\
}

static inline unsigned int __ioaddr_w (unsigned int port)		\
{									\
	if (__PORT_PCIO(port))						\
		return (unsigned int)(PCIO_BASE_w + (port));		\
	else								\
		return (unsigned int)((port));		\
}



#define DECLARE_IO(sz,fnsuffix,instr)	\
	DECLARE_DYN_IN(sz,fnsuffix,instr)

DECLARE_IO(char,b,"b")
DECLARE_IO(short,w,"h")
DECLARE_IO(int,l,"")

#undef DECLARE_IO
#undef DECLARE_DYN_IN

/*
 * Constant address IO functions
 *
 * These have to be macros for the 'J' constraint to work -
 * +/-4096 immediate operand.
 */
#define __outbc(value,port)						\
({									\
	if (__PORT_PCIO((port)))					\
		__asm__ __volatile__(					\
		"strb	%0, [%1, %2]	@ outbc"			\
		: : "r" (value), "r" (PCIO_BASE_b), "Jr" (port));	\
	else								\
		__asm__ __volatile__(					\
		"strb	%0, [%1, #0]	@ outbc"			\
		: : "r" (value), "r" (port));	                        \
})

#define __inbc(port)							\
({									\
	unsigned char result;						\
	if (__PORT_PCIO((port)))					\
		__asm__ __volatile__(					\
		"ldrb	%0, [%1, %2]	@ inbc"				\
		: "=r" (result) : "r" (PCIO_BASE_b), "Jr" (port));	\
	else								\
		__asm__ __volatile__(					\
		"ldrb	%0, [%1, #0]	@ inbc"				\
		: "=r" (result) : "r" (port));	\
	result;								\
})

#define __outwc(value,port)						\
({									\
	unsigned long v = value;					\
	if (__PORT_PCIO((port)))					\
		__asm__ __volatile__(					\
		"strh	%0, [%1, %2]	@ outwc"			\
		: : "r" (v), "r" (PCIO_BASE_w), "Jr" (port));	\
	else								\
		__asm__ __volatile__(					\
		"strh	%0, [%1, #0]	@ outwc"			\
		: : "r" (v), "r" (port));		\
})

#define __inwc(port)							\
({									\
	unsigned short result;						\
	if (__PORT_PCIO((port)))					\
		__asm__ __volatile__(					\
		"ldrh	%0, [%1, %2]	@ inwc"				\
		: "=r" (result) : "r" (PCIO_BASE_w), "Jr" (port));	\
	else								\
		__asm__ __volatile__(					\
		"ldrh	%0, [%1, #0 ]	@ inwc"				\
		: "=r" (result) : "r" (port));		\
	result;						\
})

#define __outlc(value,port) \
({									\
	unsigned long v = value;					\
	if (__PORT_PCIO((port)))					\
		__asm__ __volatile__(					\
		"strh	%0, [%1, %2]	@ outlc"			\
		: : "r" (v), "r" (PCIO_BASE_w), "Jr" (port));		\
	else								\
		__asm__ __volatile__(					\
		"strh	%0, [%1, #0]	@ outlc"			\
		: : "r" (v), "r" (port));		                \
})

#define __inlc(port)							\
({									\
	unsigned long result;						\
	if (__PORT_PCIO((port)))					\
		__asm__ __volatile__(					\
		"ldr	%0, [%1, %2]	@ inlc"				\
		: "=r" (result) : "r" (PCIO_BASE), "Jr" (port));	\
	else								\
		__asm__ __volatile__(					\
		"ldr	%0, [%1, #0]	@ inlc"				\
		: "=r" (result) : "r" (port));		                \
	result;								\
})

#define __ioaddrc(port)							\
	(__PORT_PCIO((port)) ? PCIO_BASE + (port ) : (port))

#define inb(p)	 	(__builtin_constant_p((p)) ? __inbc(p)    : __inb(p))
#define inw(p)	 	(__builtin_constant_p((p)) ? __inwc(p)    : __inw(p))
#define inl(p)	 	(__builtin_constant_p((p)) ? __inlc(p)    : __inl(p))
#define outb(v,p)	(__builtin_constant_p((p)) ? __outbc(v,p) : __outb(v,p))
#define outw(v,p)	(__builtin_constant_p((p)) ? __outwc(v,p) : __outw(v,p))
#define outl(v,p)	(__builtin_constant_p((p)) ? __outlc(v,p) : __outl(v,p))
#define __ioaddr(p)	(__builtin_constant_p((p)) ? __ioaddr(p)  : __ioaddrc(p))
/* the following macro is depreciated */
#define ioaddr(port)			__ioaddr((port))

#define insb(p,d,l)	__raw_readsb(__ioaddr_b(p),d,l)
#define insw(p,d,l)	__raw_readsw(__ioaddr_w(p),d,l)
#define insl(p,d,l)     __raw_readsl(__ioaddr_b(p),d,l)

#define outsb(p,d,l)	__raw_writesb(__ioaddr_b(p),d,l)
#define outsw(p,d,l)	__raw_writesw(__ioaddr_w(p),d,l)
#define outsl(p,d,l)    __raw_writesl(__ioaddr_b(p),d,l)


#define iomem_valid_addr(o,s)	(1)
#define iomem_to_phys(a)	(a)

#define __mem_pci(x) (x)

#endif
