/* linux/include/asm-arm/arch-bast/irq.h
 *
 * (c) 2003 Simtec Electronics
 *  Ben Dooks <ben@simtec.co.uk>
 *
 * BAST - IRQ handling
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Changelog:
 *  14-May-2003 BJD  Created file
 *  21-Jun-2003 BJD  Added checking for sub irqs
*/

#include <asm/arch/map.h>
#include <asm/io.h>

#include <asm/arch-s3c2410/S3C2410-lcd.h>

#include <asm/mach-types.h>

static inline int
fixup_irq(int irqno)
{
	unsigned long temp;
	int i;

	switch (irqno) {
	case IRQ_UART0:
	case IRQ_UART1:
	case IRQ_UART2:
	case IRQ_ADCPARENT:
		/* all these indicate sub-source, read the pending */
		temp = __raw_readl(S3C2410_VA_IRQ + 0x18);

		/* ensure masked ignored */
		temp &= ~__raw_readl(S3C2410_VA_IRQ + 0x1c);

#if 0
		llprintk("irq=%d, temp=%08x\n", irqno, temp);
#endif
		for (i = IRQ_S3CUART_RX0; i <= IRQ_ADC; i++) {
			if (temp & 1)
				return i;
			temp >>= 1;
		}

		printk(KERN_ERR "unhandled sub irq (no=%d (%d), substat=%08x, submask=%08x, mask=%08x, pend=%08x, srcpend=%08x)\n",
		   irqno,
		   __raw_readl(S3C2410_VA_IRQ + 0x14),
		   __raw_readl(S3C2410_VA_IRQ + 0x18),
		   __raw_readl(S3C2410_VA_IRQ + 0x1c),
		   __raw_readl(S3C2410_VA_IRQ + 0x08),
		   __raw_readl(S3C2410_VA_IRQ + 0x10),
		   __raw_readl(S3C2410_VA_IRQ + 0x00));

		temp = __raw_readl(S3C2410_VA_IRQ + 0x18);
		temp &= __raw_readl(S3C2410_VA_IRQ + 0x1c);

		/* we've got an unhandled interrupt... */

		switch (irqno) {
		case IRQ_UART0:
			__raw_writel(7, S3C2410_VA_IRQ + 0x18);
			return IRQ_UART0;

		case IRQ_UART1:
			__raw_writel(7<<3, S3C2410_VA_IRQ + 0x18);
			return IRQ_UART1;

		case IRQ_UART2:
			__raw_writel(7<<6, S3C2410_VA_IRQ + 0x18);
			return IRQ_UART2;

		case IRQ_ADCPARENT:
			__raw_writel(3<<9, S3C2410_VA_IRQ + 0x18);
			return IRQ_ADCPARENT;
		}

		break;

	case IRQ_ISA:
		/* IRQ_ISA is the multiplexed irq from the system CPLD */

		if (machine_is_bast()) {
			temp = __raw_readb(BAST_VA_PC104_IRQREQ) & 0xf;
			
			if (temp & 1)
				return 3;
			
			if (temp & 2)
				return 5;
			
			if (temp & 4)
				return 7;
			
			if (temp & 8)
				return 10;
		}

		break;

	case IRQ_LCD:
		temp = __raw_readl(S3C2410_LCDINTPND);
		if (temp & 2)
			return IRQ_LCD_FRAME;

		if (temp & 1)
			return IRQ_LCD_FIFO;

		printk(KERN_ERR "IRQ_LCD: invalid status=%08x\n", temp);

	default:
	}

	return irqno;
}

static __inline__ void
irq_init_irq(void)
{
	/* todo: check that we shouldn't just leave this to the
	 * architecture init sequence for our machine
	 */
}
