/* linux/include/asm-arm/arch-bast/irqs.h
 *
 * Copyright (c) 2003 Simtec Electronics
 *   Ben Dooks <ben@simtec.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Changelog:
 *  12-May-2003 BJD  Created file
 */


#ifndef __BAST_IRQ_H
#define __BAST_IRQ_H __FILE__


/* we keep the first set of CPU IRQs out of the range of
 * the ISA space, so that the PC104 has them to itself
 * and we don't end up having to do horrible things to the
 * standard ISA drivers....
 */

#define BAST_CPUIRQ_OFFSET    (16)

#define BAST_IRQ(x) ((x) + BAST_CPUIRQ_OFFSET)

/* main cpu interrupts */
#define IRQ_EINT0      BAST_IRQ(0)       /* 16 */
#define IRQ_EINT1      BAST_IRQ(1)
#define IRQ_EINT2      BAST_IRQ(2)
#define IRQ_EINT3      BAST_IRQ(3)
#define IRQ_EINT4t7    BAST_IRQ(4)
#define IRQ_EINT8t23   BAST_IRQ(5)
#define IRQ_RESERVED6  BAST_IRQ(6)
#define IRQ_BATT_FLT   BAST_IRQ(7)
#define IRQ_TICK       BAST_IRQ(8)       /* 24 */
#define IRQ_WDT        BAST_IRQ(9)
#define IRQ_TIMER0     BAST_IRQ(10)
#define IRQ_TIMER1     BAST_IRQ(11)
#define IRQ_TIMER2     BAST_IRQ(12)
#define IRQ_TIMER3     BAST_IRQ(13)
#define IRQ_TIMER4     BAST_IRQ(14)
#define IRQ_UART2      BAST_IRQ(15)
#define IRQ_LCD        BAST_IRQ(16)      /* 32 */
#define IRQ_DMA0       BAST_IRQ(17)
#define IRQ_DMA1       BAST_IRQ(18)
#define IRQ_DMA2       BAST_IRQ(19)
#define IRQ_DMA3       BAST_IRQ(20)
#define IRQ_SDI        BAST_IRQ(21)
#define IRQ_SPI0       BAST_IRQ(22)
#define IRQ_UART1      BAST_IRQ(23)
#define IRQ_RESERVED24 BAST_IRQ(24)      /* 40 */
#define IRQ_USBD       BAST_IRQ(25)
#define IRQ_USBH       BAST_IRQ(26)
#define IRQ_IIC        BAST_IRQ(27)
#define IRQ_UART0      BAST_IRQ(28)      /* 44 */
#define IRQ_SPI1       BAST_IRQ(29)
#define IRQ_RTC        BAST_IRQ(30)
#define IRQ_ADCPARENT  BAST_IRQ(31)

/* interrupts generated from the external interrupts sources */
#define IRQ_EINT4      BAST_IRQ(32)     /* 48 */
#define IRQ_EINT5      BAST_IRQ(33)
#define IRQ_EINT6      BAST_IRQ(34)
#define IRQ_EINT7      BAST_IRQ(35)
#define IRQ_EINT8      BAST_IRQ(36)
#define IRQ_EINT9      BAST_IRQ(37)
#define IRQ_EINT10     BAST_IRQ(38)
#define IRQ_EINT11     BAST_IRQ(39)
#define IRQ_EINT12     BAST_IRQ(40)
#define IRQ_EINT13     BAST_IRQ(41)
#define IRQ_EINT14     BAST_IRQ(42)
#define IRQ_EINT15     BAST_IRQ(43)
#define IRQ_EINT16     BAST_IRQ(44)
#define IRQ_EINT17     BAST_IRQ(45)
#define IRQ_EINT18     BAST_IRQ(46)
#define IRQ_EINT19     BAST_IRQ(47)
#define IRQ_EINT20     BAST_IRQ(48)     /* 64 */
#define IRQ_EINT21     BAST_IRQ(49)
#define IRQ_EINT22     BAST_IRQ(50)
#define IRQ_EINT23     BAST_IRQ(51)


#define IRQ_EINT(x)    BAST_IRQ((x >= 4) ? (IRQ_EINT4 + (x) - 4) : (BAST_IRQ(0) + (x)))

#define IRQ_LCD_FIFO   BAST_IRQ(52)
#define IRQ_LCD_FRAME  BAST_IRQ(53)

/* IRQs for the interal UARTs, and ADC
 * these need to be ordered in number of appearance in the
 * SUBSRC mask register
*/
#define IRQ_S3CUART_RX0  BAST_IRQ(54)   /* 70 */
#define IRQ_S3CUART_TX0  BAST_IRQ(55)   /* 71 */
#define IRQ_S3CUART_ERR0 BAST_IRQ(56)

#define IRQ_S3CUART_RX1  BAST_IRQ(57)
#define IRQ_S3CUART_TX1  BAST_IRQ(58)
#define IRQ_S3CUART_ERR1 BAST_IRQ(59)

#define IRQ_S3CUART_RX2  BAST_IRQ(60)
#define IRQ_S3CUART_TX2  BAST_IRQ(61)
#define IRQ_S3CUART_ERR2 BAST_IRQ(62)

#define IRQ_TC           BAST_IRQ(63)
#define IRQ_ADC          BAST_IRQ(64)

#define NR_IRQS (IRQ_ADC+1)


/* irq numbers to onboard peripherals */

#define IRQ_IDE0       IRQ_EINT16
#define IRQ_IDE1       IRQ_EINT17
#define IRQ_PCSERIAL1  IRQ_EINT15
#define IRQ_PCSERIAL2  IRQ_EINT14
#define IRQ_PCPARALLEL IRQ_EINT13
#define IRQ_ASIX       IRQ_EINT11
#define IRQ_DM9000     IRQ_EINT10
#define IRQ_ISA        IRQ_EINT9
#define IRQ_SMALERT    IRQ_EINT8

/* VR1000 serial interrupts */
#define IRQ_VR1000_SERIAL  IRQ_EINT12

#define IRQ_VR1000_SERIAL0 IRQ_EINT12
#define IRQ_VR1000_SERIAL1 IRQ_EINT13
#define IRQ_VR1000_SERIAL2 IRQ_EINT14
#define IRQ_VR1000_SERIAL3 IRQ_EINT15

/* usb over-current on EINT19 */

#endif /* __BAST_IRQ_H */
