/* linux/include/asm-arm/mach-bast/keyboard.h
 *
 * (c) 2003 Simtec Electronics
 *  Ben Dooks <ben@simtec.co.uk>
 *
 * Based on linux/include/asm-arm/arch-sa1100/keyboard.h
 *
 * BAST - CPLD control constants
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Changelog:
 *  14-Jun-2003 BJD  Created file
 *
 */

#ifndef __BAST_KEYBOARD_H
#define __BAST_KEYBOARD_H

#include <linux/config.h>
#include <asm/mach-types.h>

/* currently we don't have any keyboard code, but
 * we can implement a scanning keyboard on GPIO
 * pins or we can have an usb keyboard connected...
 */

#define kbd_disable_irq()	do { } while(0)
#define kbd_enable_irq()	do { } while(0)

extern int pckbd_setkeycode(unsigned int scancode, unsigned int keycode);
extern int pckbd_getkeycode(unsigned int scancode);
extern int pckbd_translate(unsigned char scancode, unsigned char *keycode,
			   char raw_mode);

extern char pckbd_unexpected_up(unsigned char keycode);
extern void pckbd_leds(unsigned char leds);
extern void pckbd_init_hw(void);
extern unsigned char pckbd_sysrq_xlate[128];

static inline void kbd_init_hw(void)
{
	k_setkeycode    = pckbd_setkeycode;
	k_getkeycode    = pckbd_getkeycode;
	k_translate     = pckbd_translate;
	k_unexpected_up = pckbd_unexpected_up;
	k_leds          = pckbd_leds;
#ifdef CONFIG_MAGIC_SYSRQ
	k_sysrq_key     = 0x54;
	k_sysrq_xlate   = pckbd_sysrq_xlate;
#endif
	//pckbd_init_hw();
}


/*
 * The rest of this file is to do with supporting pc_keyb.c
 */

/* resource allocation */
#define kbd_request_region() do { } while(0)
#define kbd_request_irq(handler) do { } while(0)

/* How to access the keyboard macros on this platform.  */
#define kbd_read_input() 0
#define kbd_read_status() 0
#define kbd_write_output(val) do { } while(0)
#define kbd_write_command(val) do { } while(0)

/* Some stoneage hardware needs delays after some operations.  */
#define kbd_pause() do { } while(0)

#define aux_request_irq(hand, dev_id) 0
#define aux_free_irq(dev_id) do { } while(0)


#endif  /* __BAST_KEYBOARD_H */
