/*
 *  linux/include/asm-arm/arch-bast/time.h
 *
 *  Copyright (C) SAMSUNG ELECTRONICS
 *                      SW.LEE <hitchcar@sec.samsung.com>
 *
 *  Copyright (C) 2003 Simtec Electronics <linux@simtec.co.uk>
 *    Modifications to original S3C2410 for BAST
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <asm/system.h>
#include <asm/leds.h>

#include <asm/io.h>
#include <asm/arch/map.h>
#include <asm/arch-s3c2410/S3C2410-timer.h>

extern unsigned long (*gettimeoffset)(void);

/* we assume that the timer has already been configured
 * by the bootloader to use the TCNT1 input.
 *
 * we have 12MHz presented at TCNT1, which we can reasonably
 * make any HZ value down to 183Hz....
 */


/* timer_startval = the value of the counter counts down from
 */
static unsigned long timer_startval;

#ifdef CONFIG_BAST_RTC
extern void bast_rtc_check();
#else
#define bast_rtc_check()
#endif

/***
 * Returns microsecond  since last clock interrupt.  Note that interrupts
 * will have been disabled by do_gettimeoffset()
 * IRQs are disabled before entering here from do_gettimeofday()
 */
static unsigned long bast_gettimeoffset (void)
{
        unsigned long elapsed;
	unsigned long usec;
	unsigned long tcnt;

	/*
	 * tick is microsecond unit
	 * long tick = (1000000 + HZ/2) / HZ;	 timer interrupt period
	 * kernel/timer.c
	 */

	tcnt = __raw_readl(S3C2410_TCNTO(4));

	/* note, we don't check to see if an irq has gone
	 * off and hasn't been handled... */

	elapsed = timer_startval - tcnt;

	/*
	 *  A SIMPLE RATIO
	 * (elapsed / rTCNTB4) =  ( usec / tick )
	 */

	usec = (elapsed * tick) / timer_startval ;
        return usec;
}


/*
 * IRQ handler for the timer
 */
static void bast_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	long flags;

	do_leds();
	save_flags_cli( flags );
	do_timer(regs);
	restore_flags( flags );
	do_set_rtc();
	bast_rtc_check();
	do_profile(regs);

}

/*
 * Set up timer interrupt, and return the current time in seconds.
 */

#define dbg(x) printk x

static inline void setup_timer (void)
{
        unsigned long tcon;
	unsigned long tcnt;

	dbg((__FUNCTION__ ": TCNTB4 at %08x\n", S3C2410_TCNTB(4)));

        gettimeoffset = bast_gettimeoffset;
	timer_irq.handler = bast_timer_interrupt;

	tcon = __raw_readl(S3C2410_TCON);
	tcnt = (12*(1000*1000)) / HZ;

	dbg((__FUNCTION__ ": tcon=%08lx, tcnt should be %04lx\n", tcon, tcnt));

	/* check to see if timer is within 16bit range... */
	if (tcnt > 0xffff) {
	  dbg((__FUNCTION__ ": tcnt too large\n"));
	  panic("setup_timer: HZ is too small, cannot configure timer!");
	  return;
	}

	dbg((__FUNCTION__ ": about to configure timer4\n"));
	timer_startval = tcnt;
	__raw_writel(tcnt, S3C2410_TCNTB(4));

	dbg((__FUNCTION__ ": stopping timer\n"));

	/* ensure timer is stopped... */

	tcon &= ~(7<<20);
	tcon |= S3C2410_TCON_T4RELOAD;
	tcon |= S3C2410_TCON_T4MANUALUPD;

	__raw_writel(tcon, S3C2410_TCON);
	__raw_writel(tcnt, S3C2410_TCNTB(4));
	__raw_writel(tcnt, S3C2410_TCMPB(4));

	dbg((__FUNCTION__ ": adding interrupt\n"));
	setup_arm_irq(IRQ_TIMER4, &timer_irq);

	dbg((__FUNCTION__ ": starting timer\n"));

	/* start the timer running */
	tcon |= S3C2410_TCON_T4START;
	tcon &= ~S3C2410_TCON_T4MANUALUPD;
	__raw_writel(tcon, S3C2410_TCON);
}
